@@ -2,6 +2,7 @@
 
 use alloc::{string::ToString, vec::Vec};
 use core::{marker::PhantomData, time::Duration};
+use core_affinity::CoreId;
 use serde::{de::DeserializeOwned, Serialize};
 
 #[cfg(feature = "std")]
@@ -13,18 +14,20 @@ use crate::bolts::{
     shmem::StdShMemProvider,
 };
 
+#[cfg(feature = "std")]
+use std::net::{SocketAddr, ToSocketAddrs};
+
 use crate::{
     bolts::{
         llmp::{self, Flags, LlmpClientDescription, LlmpSender, Tag},
         shmem::ShMemProvider,
     },
-    corpus::CorpusScheduler,
-    events::{BrokerEventResult, Event, EventManager},
+    events::{BrokerEventResult, Event, EventFirer, EventManager, EventProcessor, EventRestarter},
+    executors::Executor,
     executors::ExitKind,
-    executors::{Executor, HasObservers},
+    fuzzer::{IfInteresting, IsInteresting},
     inputs::Input,
     observers::ObserversTuple,
-    state::IfInteresting,
     stats::Stats,
     Error,
 };
@@ -36,14 +39,17 @@ use crate::bolts::{
 };
 
 #[cfg(all(feature = "std", windows))]
-use crate::utils::startable_self;
+use crate::bolts::os::startable_self;
 
 #[cfg(all(feature = "std", unix))]
-use crate::utils::{fork, ForkResult};
+use crate::bolts::os::{fork, ForkResult};
 
-#[cfg(all(feature = "std", target_os = "android"))]
+#[cfg(all(target_os = "android", feature = "std"))]
 use crate::bolts::os::ashmem_server::AshmemService;
 
+#[cfg(feature = "std")]
+use typed_builder::TypedBuilder;
+
 /// Forward this to the client
 const _LLMP_TAG_EVENT_TO_CLIENT: llmp::Tag = 0x2C11E471;
 /// Only handle this in the broker
@@ -57,10 +63,10 @@ const _LLMP_TAG_NO_RESTART: llmp::Tag = 0x57A7EE71;
 /// An [`EventManager`] that forwards all events to other attached fuzzers on shared maps or via tcp,
 /// using low-level message passing, [`crate::bolts::llmp`].
 #[derive(Debug)]
-pub struct LlmpEventManager<I, S, SP, ST>
+pub struct LlmpEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I>,
+    OT: ObserversTuple,
     SP: ShMemProvider + 'static,
     ST: Stats,
     //CE: CustomEvent<I>,
@@ -70,18 +76,18 @@ where
     #[cfg(feature = "llmp_compression")]
     compressor: GzipCompressor,
 
-    phantom: PhantomData<(I, S)>,
+    phantom: PhantomData<(I, OT, S)>,
 }
 
 /// The minimum buffer size at which to compress LLMP IPC messages.
 #[cfg(feature = "llmp_compression")]
 const COMPRESS_THRESHOLD: usize = 1024;
 
-impl<I, S, SP, ST> Drop for LlmpEventManager<I, S, SP, ST>
+impl<I, OT, S, SP, ST> Drop for LlmpEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I>,
-    SP: ShMemProvider,
+    OT: ObserversTuple,
+    SP: ShMemProvider + 'static,
     ST: Stats,
 {
     /// LLMP clients will have to wait until their pages are mapped by somebody.
@@ -90,11 +96,11 @@ where
     }
 }
 
-impl<I, S, SP, ST> LlmpEventManager<I, S, SP, ST>
+impl<I, OT, S, SP, ST> LlmpEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I>,
-    SP: ShMemProvider,
+    OT: ObserversTuple,
+    SP: ShMemProvider + 'static,
     ST: Stats,
 {
     /// Create llmp on a port
@@ -167,6 +173,19 @@ where
         matches!(self.llmp, llmp::LlmpConnection::IsBroker { broker: _ })
     }
 
+    #[cfg(feature = "std")]
+    pub fn connect_b2b<A>(&mut self, addr: A) -> Result<(), Error>
+    where
+        A: ToSocketAddrs,
+    {
+        match &mut self.llmp {
+            llmp::LlmpConnection::IsBroker { broker } => broker.connect_b2b(addr),
+            llmp::LlmpConnection::IsClient { client: _ } => Err(Error::IllegalState(
+                "Called broker loop in the client".into(),
+            )),
+        }
+    }
+
     /// Run forever in the broker
     pub fn broker_loop(&mut self) -> Result<(), Error> {
         match &mut self.llmp {
@@ -293,18 +312,16 @@ where
 
     // Handle arriving events in the client
     #[allow(clippy::unused_self)]
-    fn handle_in_client<CS, E, OT>(
+    fn handle_in_client<E, Z>(
         &mut self,
+        fuzzer: &mut Z,
+        _executor: &mut E,
         state: &mut S,
         _sender_id: u32,
         event: Event<I>,
-        _executor: &mut E,
-        scheduler: &CS,
     ) -> Result<(), Error>
     where
-        CS: CorpusScheduler<I, S>,
-        E: Executor<I> + HasObservers<OT>,
-        OT: ObserversTuple,
+        Z: IfInteresting<I, S> + IsInteresting<I, OT, S>,
     {
         match event {
             Event::NewTestcase {
@@ -322,9 +339,10 @@ where
 
                 let observers: OT = postcard::from_bytes(&observers_buf)?;
                 // TODO include ExitKind in NewTestcase
-                let is_interesting = state.is_interesting(&input, &observers, &ExitKind::Ok)?;
-                if state
-                    .add_if_interesting(&input, is_interesting, scheduler)?
+                let is_interesting =
+                    fuzzer.is_interesting(state, &input, &observers, &ExitKind::Ok)?;
+                if fuzzer
+                    .add_if_interesting(state, &input, is_interesting)?
                     .is_some()
                 {
                     #[cfg(feature = "std")]
@@ -340,12 +358,49 @@ where
     }
 }
 
-impl<I, S, SP, ST> EventManager<I, S> for LlmpEventManager<I, S, SP, ST>
+impl<I, OT, S, SP, ST> EventFirer<I, S> for LlmpEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I>,
+    OT: ObserversTuple,
     SP: ShMemProvider,
-    ST: Stats, //CE: CustomEvent<I>,
+    ST: Stats,
+    //CE: CustomEvent<I>,
+{
+    #[cfg(feature = "llmp_compression")]
+    fn fire(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
+        let serialized = postcard::to_allocvec(&event)?;
+        let flags: Flags = LLMP_FLAG_INITIALIZED;
+
+        match self.compressor.compress(&serialized)? {
+            Some(comp_buf) => {
+                self.llmp.send_buf_with_flags(
+                    LLMP_TAG_EVENT_TO_BOTH,
+                    &comp_buf,
+                    flags | LLMP_FLAG_COMPRESSED,
+                )?;
+            }
+            None => {
+                self.llmp.send_buf(LLMP_TAG_EVENT_TO_BOTH, &serialized)?;
+            }
+        }
+        Ok(())
+    }
+
+    #[cfg(not(feature = "llmp_compression"))]
+    fn fire(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
+        let serialized = postcard::to_allocvec(&event)?;
+        self.llmp.send_buf(LLMP_TAG_EVENT_TO_BOTH, &serialized)?;
+        Ok(())
+    }
+}
+
+impl<I, OT, S, SP, ST> EventRestarter<S> for LlmpEventManager<I, OT, S, SP, ST>
+where
+    I: Input,
+    OT: ObserversTuple,
+    SP: ShMemProvider,
+    ST: Stats,
+    //CE: CustomEvent<I>,
 {
     /// The llmp client needs to wait until a broker mapped all pages, before shutting down.
     /// Otherwise, the OS may already have removed the shared maps,
@@ -355,18 +410,18 @@ where
             client.await_save_to_unmap_blocking();
         }
     }
+}
 
-    fn process<CS, E, OT>(
-        &mut self,
-        state: &mut S,
-        executor: &mut E,
-        scheduler: &CS,
-    ) -> Result<usize, Error>
-    where
-        CS: CorpusScheduler<I, S>,
-        E: Executor<I> + HasObservers<OT>,
-        OT: ObserversTuple,
-    {
+impl<E, I, OT, S, SP, ST, Z> EventProcessor<E, S, Z> for LlmpEventManager<I, OT, S, SP, ST>
+where
+    SP: ShMemProvider,
+    ST: Stats,
+    E: Executor<I>,
+    I: Input,
+    OT: ObserversTuple,
+    Z: IfInteresting<I, S> + IsInteresting<I, OT, S>, //CE: CustomEvent<I>,
+{
+    fn process(&mut self, fuzzer: &mut Z, state: &mut S, executor: &mut E) -> Result<usize, Error> {
         // TODO: Get around local event copy by moving handle_in_client
         let mut events = vec![];
         match &mut self.llmp {
@@ -397,49 +452,34 @@ where
         };
         let count = events.len();
         events.drain(..).try_for_each(|(sender_id, event)| {
-            self.handle_in_client(state, sender_id, event, executor, scheduler)
+            self.handle_in_client(fuzzer, executor, state, sender_id, event)
         })?;
         Ok(count)
     }
+}
 
-    #[cfg(feature = "llmp_compression")]
-    fn fire(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
-        let serialized = postcard::to_allocvec(&event)?;
-        let flags: Flags = LLMP_FLAG_INITIALIZED;
-
-        match self.compressor.compress(&serialized)? {
-            Some(comp_buf) => {
-                self.llmp.send_buf_with_flags(
-                    LLMP_TAG_EVENT_TO_BOTH,
-                    &comp_buf,
-                    flags | LLMP_FLAG_COMPRESSED,
-                )?;
-            }
-            None => {
-                self.llmp.send_buf(LLMP_TAG_EVENT_TO_BOTH, &serialized)?;
-            }
-        }
-        Ok(())
-    }
-
-    #[cfg(not(feature = "llmp_compression"))]
-    fn fire(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
-        let serialized = postcard::to_allocvec(&event)?;
-        self.llmp.send_buf(LLMP_TAG_EVENT_TO_BOTH, &serialized)?;
-        Ok(())
-    }
+impl<E, I, OT, S, SP, ST, Z> EventManager<E, I, S, Z> for LlmpEventManager<I, OT, S, SP, ST>
+where
+    SP: ShMemProvider,
+    ST: Stats,
+    E: Executor<I>,
+    I: Input,
+    OT: ObserversTuple,
+    Z: IfInteresting<I, S> + IsInteresting<I, OT, S>, //CE: CustomEvent<I>,
+{
 }
 
 /// Serialize the current state and corpus during an executiont to bytes.
 /// On top, add the current llmp event manager instance to be restored
 /// This method is needed when the fuzzer run crashes and has to restart.
-pub fn serialize_state_mgr<I, S, SP, ST>(
+pub fn serialize_state_mgr<I, OT, S, SP, ST>(
     state: &S,
-    mgr: &LlmpEventManager<I, S, SP, ST>,
+    mgr: &LlmpEventManager<I, OT, S, SP, ST>,
 ) -> Result<Vec<u8>, Error>
 where
     I: Input,
-    S: Serialize + IfInteresting<I>,
+    OT: ObserversTuple,
+    S: Serialize,
     SP: ShMemProvider,
     ST: Stats,
 {
@@ -448,13 +488,14 @@ where
 
 /// Deserialize the state and corpus tuple, previously serialized with `serialize_state_corpus(...)`
 #[allow(clippy::type_complexity)]
-pub fn deserialize_state_mgr<I, S, SP, ST>(
+pub fn deserialize_state_mgr<I, OT, S, SP, ST>(
     shmem_provider: SP,
     state_corpus_serialized: &[u8],
-) -> Result<(S, LlmpEventManager<I, S, SP, ST>), Error>
+) -> Result<(S, LlmpEventManager<I, OT, S, SP, ST>), Error>
 where
     I: Input,
-    S: DeserializeOwned + IfInteresting<I>,
+    OT: ObserversTuple,
+    S: DeserializeOwned,
     SP: ShMemProvider,
     ST: Stats,
 {
@@ -467,32 +508,49 @@ where
 
 /// A manager that can restart on the fly, storing states in-between (in `on_resatrt`)
 #[derive(Debug)]
-pub struct LlmpRestartingEventManager<I, S, SP, ST>
+pub struct LlmpRestartingEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I>,
+    OT: ObserversTuple,
     SP: ShMemProvider + 'static,
     ST: Stats,
     //CE: CustomEvent<I>,
 {
     /// The embedded llmp event manager
-    llmp_mgr: LlmpEventManager<I, S, SP, ST>,
+    llmp_mgr: LlmpEventManager<I, OT, S, SP, ST>,
     /// The sender to serialize the state for the next runner
     sender: LlmpSender<SP>,
 }
 
-impl<I, S, SP, ST> EventManager<I, S> for LlmpRestartingEventManager<I, S, SP, ST>
+impl<I, OT, S, SP, ST> EventFirer<I, S> for LlmpRestartingEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I> + Serialize,
+    OT: ObserversTuple,
+    S: Serialize,
     SP: ShMemProvider,
-    ST: Stats, //CE: CustomEvent<I>,
+    ST: Stats,
+    //CE: CustomEvent<I>,
+{
+    fn fire(&mut self, state: &mut S, event: Event<I>) -> Result<(), Error> {
+        // Check if we are going to crash in the event, in which case we store our current state for the next runner
+        self.llmp_mgr.fire(state, event)
+    }
+}
+
+impl<I, OT, S, SP, ST> EventRestarter<S> for LlmpRestartingEventManager<I, OT, S, SP, ST>
+where
+    I: Input,
+    OT: ObserversTuple,
+    S: Serialize,
+    SP: ShMemProvider,
+    ST: Stats,
+    //CE: CustomEvent<I>,
 {
     /// The llmp client needs to wait until a broker mapped all pages, before shutting down.
     /// Otherwise, the OS may already have removed the shared maps,
     #[inline]
     fn await_restart_safe(&mut self) {
-        self.llmp_mgr.await_restart_safe();
+        self.llmp_mgr.await_restart_safe()
     }
 
     /// Reset the single page (we reuse it over and over from pos 0), then send the current state to the next runner.
@@ -503,25 +561,36 @@ where
         self.sender
             .send_buf(_LLMP_TAG_RESTART, &state_corpus_serialized)
     }
+}
 
-    fn process<CS, E, OT>(
-        &mut self,
-        state: &mut S,
-        executor: &mut E,
-        scheduler: &CS,
-    ) -> Result<usize, Error>
-    where
-        CS: CorpusScheduler<I, S>,
-        E: Executor<I> + HasObservers<OT>,
-        OT: ObserversTuple,
-    {
-        self.llmp_mgr.process(state, executor, scheduler)
+impl<E, I, OT, S, SP, ST, Z> EventProcessor<E, S, Z>
+    for LlmpRestartingEventManager<I, OT, S, SP, ST>
+where
+    E: Executor<I>,
+    I: Input,
+    Z: IfInteresting<I, S> + IsInteresting<I, OT, S>,
+    OT: ObserversTuple,
+    SP: ShMemProvider + 'static,
+    ST: Stats,
+    //CE: CustomEvent<I>,
+{
+    fn process(&mut self, fuzzer: &mut Z, state: &mut S, executor: &mut E) -> Result<usize, Error> {
+        self.llmp_mgr.process(fuzzer, state, executor)
     }
+}
 
-    fn fire(&mut self, state: &mut S, event: Event<I>) -> Result<(), Error> {
-        // Check if we are going to crash in the event, in which case we store our current state for the next runner
-        self.llmp_mgr.fire(state, event)
-    }
+impl<E, I, OT, S, SP, ST, Z> EventManager<E, I, S, Z>
+    for LlmpRestartingEventManager<I, OT, S, SP, ST>
+where
+    E: Executor<I>,
+    I: Input,
+    S: Serialize,
+    Z: IfInteresting<I, S> + IsInteresting<I, OT, S>,
+    OT: ObserversTuple,
+    SP: ShMemProvider + 'static,
+    ST: Stats,
+    //CE: CustomEvent<I>,
+{
 }
 
 /// The llmp connection from the actual fuzzer to the process supervising it
@@ -530,15 +599,16 @@ const _ENV_FUZZER_RECEIVER: &str = &"_AFL_ENV_FUZZER_RECEIVER";
 /// The llmp (2 way) connection from a fuzzer to the broker (broadcasting all other fuzzer messages)
 const _ENV_FUZZER_BROKER_CLIENT_INITIAL: &str = &"_AFL_ENV_FUZZER_BROKER_CLIENT";
 
-impl<I, S, SP, ST> LlmpRestartingEventManager<I, S, SP, ST>
+impl<I, OT, S, SP, ST> LlmpRestartingEventManager<I, OT, S, SP, ST>
 where
     I: Input,
-    S: IfInteresting<I>,
-    SP: ShMemProvider,
-    ST: Stats, //CE: CustomEvent<I>,
+    OT: ObserversTuple,
+    SP: ShMemProvider + 'static,
+    ST: Stats,
+    //CE: CustomEvent<I>,
 {
     /// Create a new runner, the executed child doing the actual fuzzing.
-    pub fn new(llmp_mgr: LlmpEventManager<I, S, SP, ST>, sender: LlmpSender<SP>) -> Self {
+    pub fn new(llmp_mgr: LlmpEventManager<I, OT, S, SP, ST>, sender: LlmpSender<SP>) -> Self {
         Self { llmp_mgr, sender }
     }
 
@@ -553,160 +623,260 @@ where
     }
 }
 
+/// The kind of manager we're creating right now
+#[derive(Debug, Clone, Copy)]
+pub enum ManagerKind {
+    /// Any kind will do
+    Any,
+    /// A client, getting messages from a local broker.
+    Client { cpu_core: Option<CoreId> },
+    /// A [`LlmpBroker`], forwarding the packets of local clients.
+    Broker,
+}
+
 /// Sets up a restarting fuzzer, using the [`StdShMemProvider`], and standard features.
 /// The restarting mgr is a combination of restarter and runner, that can be used on systems with and without `fork` support.
 /// The restarter will spawn a new process each time the child crashes or timeouts.
 #[cfg(feature = "std")]
 #[allow(clippy::type_complexity)]
-pub fn setup_restarting_mgr_std<I, S, ST>(
-    //mgr: &mut LlmpEventManager<I, S, SH, ST>,
+pub fn setup_restarting_mgr_std<I, OT, S, ST>(
     stats: ST,
     broker_port: u16,
 ) -> Result<
     (
         Option<S>,
-        LlmpRestartingEventManager<I, S, StdShMemProvider, ST>,
+        LlmpRestartingEventManager<I, OT, S, StdShMemProvider, ST>,
     ),
     Error,
 >
 where
     I: Input,
-    S: DeserializeOwned + IfInteresting<I>,
-    ST: Stats,
+    S: DeserializeOwned,
+    ST: Stats + Clone,
+    OT: ObserversTuple,
+    S: DeserializeOwned,
 {
     #[cfg(target_os = "android")]
     AshmemService::start().expect("Error starting Ashmem Service");
 
-    setup_restarting_mgr(StdShMemProvider::new()?, stats, broker_port)
+    RestartingMgr::builder()
+        .shmem_provider(StdShMemProvider::new()?)
+        .stats(stats)
+        .broker_port(broker_port)
+        .build()
+        .launch()
 }
 
-/// A restarting state is a combination of restarter and runner, that can be used on systems with and without `fork` support.
-/// The restarter will start a new process each time the child crashes or timeouts.
+/// Provides a `builder` which can be used to build a [`RestartingMgr`], which is a combination of a
+/// `restarter` and `runner`, that can be used on systems both with and without `fork` support. The
+/// `restarter` will start a new process each time the child crashes or times out.
 #[cfg(feature = "std")]
-#[allow(
-    clippy::unnecessary_operation,
-    clippy::type_complexity,
-    clippy::similar_names
-)] // for { mgr = LlmpEventManager... }
-pub fn setup_restarting_mgr<I, S, SP, ST>(
-    mut shmem_provider: SP,
-    //mgr: &mut LlmpEventManager<I, S, SH, ST>,
-    stats: ST,
-    broker_port: u16,
-) -> Result<(Option<S>, LlmpRestartingEventManager<I, S, SP, ST>), Error>
+#[allow(clippy::default_trait_access)]
+#[derive(TypedBuilder, Debug)]
+pub struct RestartingMgr<I, OT, S, SP, ST>
 where
     I: Input,
-    S: DeserializeOwned + IfInteresting<I>,
-    SP: ShMemProvider,
+    OT: ObserversTuple,
+    S: DeserializeOwned,
+    SP: ShMemProvider + 'static,
     ST: Stats,
+    //CE: CustomEvent<I>,
 {
-    let mut mgr =
-        LlmpEventManager::<I, S, SP, ST>::new_on_port(shmem_provider.clone(), stats, broker_port)?;
+    /// The shared memory provider to use for the broker or client spawned by the restarting
+    /// manager.
+    shmem_provider: SP,
+    /// The stats to use
+    stats: ST,
+    /// The broker port to use
+    #[builder(default = 1337_u16)]
+    broker_port: u16,
+    /// The address to connect to
+    #[builder(default = None)]
+    remote_broker_addr: Option<SocketAddr>,
+    /// The type of manager to build
+    #[builder(default = ManagerKind::Any)]
+    kind: ManagerKind,
+    #[builder(setter(skip), default = PhantomData {})]
+    _phantom: PhantomData<(I, OT, S)>,
+}
 
-    // We start ourself as child process to actually fuzz
-    let (sender, mut receiver, mut new_shmem_provider) = if std::env::var(_ENV_FUZZER_SENDER)
+#[cfg(feature = "std")]
+#[allow(clippy::type_complexity)]
+#[allow(clippy::too_many_lines)]
+impl<I, OT, S, SP, ST> RestartingMgr<I, OT, S, SP, ST>
+where
+    I: Input,
+    OT: ObserversTuple,
+    S: DeserializeOwned,
+    SP: ShMemProvider,
+    ST: Stats + Clone,
+{
+    /// Launch the restarting manager
+    pub fn launch(
+        &mut self,
+    ) -> Result<(Option<S>, LlmpRestartingEventManager<I, OT, S, SP, ST>), Error> {
+        let mut mgr = LlmpEventManager::<I, OT, S, SP, ST>::new_on_port(
+            self.shmem_provider.clone(),
+            self.stats.clone(),
+            self.broker_port,
+        )?;
+
+        // We start ourself as child process to actually fuzz
+        let (sender, mut receiver, new_shmem_provider, core_id) = if std::env::var(
+            _ENV_FUZZER_SENDER,
+        )
         .is_err()
-    {
-        if mgr.is_broker() {
-            // Yep, broker. Just loop here.
-            println!("Doing broker things. Run this tool again to start fuzzing in a client.");
-            mgr.broker_loop()?;
-            return Err(Error::ShuttingDown);
-        }
+        {
+            // We get here if we are on Unix, or we are a broker on Windows.
+            let core_id = if mgr.is_broker() {
+                match self.kind {
+                    ManagerKind::Broker | ManagerKind::Any => {
+                        // Yep, broker. Just loop here.
+                        println!(
+                            "Doing broker things. Run this tool again to start fuzzing in a client."
+                        );
 
-        // We are the fuzzer respawner in a llmp client
-        mgr.to_env(_ENV_FUZZER_BROKER_CLIENT_INITIAL);
+                        if let Some(remote_broker_addr) = self.remote_broker_addr {
+                            println!("B2b: Connecting to {:?}", &remote_broker_addr);
+                            mgr.connect_b2b(remote_broker_addr)?;
+                        };
 
-        // First, create a channel from the fuzzer (sender) to us (receiver) to report its state for restarts.
-        let sender = { LlmpSender::new(shmem_provider.clone(), 0, false)? };
-
-        let map = { shmem_provider.clone_ref(&sender.out_maps.last().unwrap().shmem)? };
-        let receiver = LlmpReceiver::on_existing_map(shmem_provider.clone(), map, None)?;
-        // Store the information to a map.
-        sender.to_env(_ENV_FUZZER_SENDER)?;
-        receiver.to_env(_ENV_FUZZER_RECEIVER)?;
-
-        let mut ctr: u64 = 0;
-        // Client->parent loop
-        loop {
-            dbg!("Spawning next client (id {})", ctr);
-
-            // On Unix, we fork (todo: measure if that is actually faster.)
-            #[cfg(unix)]
-            let child_status = match unsafe { fork() }? {
-                ForkResult::Parent(handle) => handle.status(),
-                ForkResult::Child => break (sender, receiver, shmem_provider),
+                        mgr.broker_loop()?;
+                        return Err(Error::ShuttingDown);
+                    }
+                    ManagerKind::Client { cpu_core: _ } => {
+                        return Err(Error::IllegalState(
+                            "Tried to start a client, but got a broker".to_string(),
+                        ));
+                    }
+                }
+            } else {
+                match self.kind {
+                    ManagerKind::Broker => {
+                        return Err(Error::IllegalState(
+                            "Tried to start a broker, but got a client".to_string(),
+                        ));
+                    }
+                    ManagerKind::Client { cpu_core } => cpu_core,
+                    ManagerKind::Any => None,
+                }
             };
 
-            // On windows, we spawn ourself again
-            #[cfg(windows)]
-            let child_status = startable_self()?.status()?;
-
-            if unsafe { read_volatile(addr_of!((*receiver.current_recv_map.page()).size_used)) }
-                == 0
-            {
-                #[cfg(unix)]
-                if child_status == 137 {
-                    // Out of Memory, see https://tldp.org/LDP/abs/html/exitcodes.html
-                    // and https://github.com/AFLplusplus/LibAFL/issues/32 for discussion.
-                    panic!("Fuzzer-respawner: The fuzzed target crashed with an out of memory error! Fix your harness, or switch to another executor (for example, a forkserver).");
-                }
-
-                // Storing state in the last round did not work
-                panic!("Fuzzer-respawner: Storing state in crashed fuzzer instance did not work, no point to spawn the next client! (Child exited with: {})", child_status);
+            if let Some(core_id) = core_id {
+                println!("Setting core affinity to {:?}", core_id);
+                core_affinity::set_for_current(core_id);
             }
 
-            ctr = ctr.wrapping_add(1);
+            // We are the fuzzer respawner in a llmp client
+            mgr.to_env(_ENV_FUZZER_BROKER_CLIENT_INITIAL);
+
+            // First, create a channel from the fuzzer (sender) to us (receiver) to report its state for restarts.
+            let sender = { LlmpSender::new(self.shmem_provider.clone(), 0, false)? };
+
+            let map = {
+                self.shmem_provider
+                    .clone_ref(&sender.out_maps.last().unwrap().shmem)?
+            };
+            let receiver = LlmpReceiver::on_existing_map(self.shmem_provider.clone(), map, None)?;
+            // Store the information to a map.
+            sender.to_env(_ENV_FUZZER_SENDER)?;
+            receiver.to_env(_ENV_FUZZER_RECEIVER)?;
+
+            let mut ctr: u64 = 0;
+            // Client->parent loop
+            loop {
+                dbg!("Spawning next client (id {})", ctr);
+
+                // On Unix, we fork
+                #[cfg(unix)]
+                let child_status = {
+                    self.shmem_provider.pre_fork()?;
+                    match unsafe { fork() }? {
+                        ForkResult::Parent(handle) => {
+                            self.shmem_provider.post_fork(false)?;
+                            handle.status()
+                        }
+                        ForkResult::Child => {
+                            self.shmem_provider.post_fork(true)?;
+                            break (sender, receiver, self.shmem_provider.clone(), core_id);
+                        }
+                    }
+                };
+
+                // On windows, we spawn ourself again
+                #[cfg(windows)]
+                let child_status = startable_self()?.status()?;
+
+                if unsafe { read_volatile(addr_of!((*receiver.current_recv_map.page()).size_used)) }
+                    == 0
+                {
+                    #[cfg(unix)]
+                    if child_status == 137 {
+                        // Out of Memory, see https://tldp.org/LDP/abs/html/exitcodes.html
+                        // and https://github.com/AFLplusplus/LibAFL/issues/32 for discussion.
+                        panic!("Fuzzer-respawner: The fuzzed target crashed with an out of memory error! Fix your harness, or switch to another executor (for example, a forkserver).");
+                    }
+
+                    // Storing state in the last round did not work
+                    panic!("Fuzzer-respawner: Storing state in crashed fuzzer instance did not work, no point to spawn the next client! (Child exited with: {})", child_status);
+                }
+
+                ctr = ctr.wrapping_add(1);
+            }
+        } else {
+            // We are the newly started fuzzing instance (i.e. on Windows), first, connect to our own restore map.
+            // We get here *only on Windows*, if we were started by a restarting fuzzer.
+            // A sender and a receiver for single communication
+            (
+                LlmpSender::on_existing_from_env(self.shmem_provider.clone(), _ENV_FUZZER_SENDER)?,
+                LlmpReceiver::on_existing_from_env(
+                    self.shmem_provider.clone(),
+                    _ENV_FUZZER_RECEIVER,
+                )?,
+                self.shmem_provider.clone(),
+                None,
+            )
+        };
+
+        if let Some(core_id) = core_id {
+            core_affinity::set_for_current(core_id);
         }
-    } else {
-        // We are the newly started fuzzing instance, first, connect to our own restore map.
-        // A sender and a receiver for single communication
-        // Clone so we get a new connection to the AshmemServer if we are using
-        // ServedShMemProvider
-        shmem_provider.post_fork();
-        (
-            LlmpSender::on_existing_from_env(shmem_provider.clone(), _ENV_FUZZER_SENDER)?,
-            LlmpReceiver::on_existing_from_env(shmem_provider.clone(), _ENV_FUZZER_RECEIVER)?,
-            shmem_provider,
-        )
-    };
 
-    new_shmem_provider.post_fork();
+        println!("We're a client, let's fuzz :)");
 
-    println!("We're a client, let's fuzz :)");
+        for (var, val) in std::env::vars() {
+            println!("ENV VARS: {:?}: {:?}", var, val);
+        }
 
-    for (var, val) in std::env::vars() {
-        println!("ENV VARS: {:?}: {:?}", var, val);
+        // If we're restarting, deserialize the old state.
+        let (state, mut mgr) = match receiver.recv_buf()? {
+            None => {
+                println!("First run. Let's set it all up");
+                // Mgr to send and receive msgs from/to all other fuzzer instances
+                let client_mgr = LlmpEventManager::<I, OT, S, SP, ST>::existing_client_from_env(
+                    new_shmem_provider,
+                    _ENV_FUZZER_BROKER_CLIENT_INITIAL,
+                )?;
+
+                (None, LlmpRestartingEventManager::new(client_mgr, sender))
+            }
+            // Restoring from a previous run, deserialize state and corpus.
+            Some((_sender, _tag, msg)) => {
+                println!("Subsequent run. Let's load all data from shmem (received {} bytes from previous instance)", msg.len());
+                let (state, mgr): (S, LlmpEventManager<I, OT, S, SP, ST>) =
+                    deserialize_state_mgr(new_shmem_provider, &msg)?;
+
+                (Some(state), LlmpRestartingEventManager::new(mgr, sender))
+            }
+        };
+        // We reset the sender, the next sender and receiver (after crash) will reuse the page from the initial message.
+        unsafe { mgr.sender_mut().reset() };
+        /* TODO: Not sure if this is needed
+        // We commit an empty NO_RESTART message to this buf, against infinite loops,
+        // in case something crashes in the fuzzer.
+        sender.send_buf(_LLMP_TAG_NO_RESTART, []);
+        */
+
+        Ok((state, mgr))
     }
-
-    // If we're restarting, deserialize the old state.
-    let (state, mut mgr) = match receiver.recv_buf()? {
-        None => {
-            println!("First run. Let's set it all up");
-            // Mgr to send and receive msgs from/to all other fuzzer instances
-            let client_mgr = LlmpEventManager::<I, S, SP, ST>::existing_client_from_env(
-                new_shmem_provider,
-                _ENV_FUZZER_BROKER_CLIENT_INITIAL,
-            )?;
-
-            (None, LlmpRestartingEventManager::new(client_mgr, sender))
-        }
-        // Restoring from a previous run, deserialize state and corpus.
-        Some((_sender, _tag, msg)) => {
-            println!("Subsequent run. Let's load all data from shmem (received {} bytes from previous instance)", msg.len());
-            let (state, mgr): (S, LlmpEventManager<I, S, SP, ST>) =
-                deserialize_state_mgr(new_shmem_provider, &msg)?;
-
-            (Some(state), LlmpRestartingEventManager::new(mgr, sender))
-        }
-    };
-    // We reset the sender, the next sender and receiver (after crash) will reuse the page from the initial message.
-    unsafe { mgr.sender_mut().reset() };
-    /* TODO: Not sure if this is needed
-    // We commit an empty NO_RESTART message to this buf, against infinite loops,
-    // in case something crashes in the fuzzer.
-    sender.send_buf(_LLMP_TAG_NO_RESTART, []);
-    */
-
-    Ok((state, mgr))
 }
