@@ -1,20 +1,16 @@
 //! A very simple event manager, that just supports log outputs, but no multiprocessing
 use alloc::{string::ToString, vec::Vec};
-use core::marker::PhantomData;
 
 use crate::{
-    corpus::CorpusScheduler,
-    events::{BrokerEventResult, Event, EventManager},
-    executors::{Executor, HasObservers},
+    events::{BrokerEventResult, Event, EventFirer, EventManager, EventProcessor, EventRestarter},
     inputs::Input,
-    observers::ObserversTuple,
     stats::Stats,
     Error,
 };
 
 /// A simple, single-threaded event manager that just logs
 #[derive(Clone, Debug)]
-pub struct SimpleEventManager<I, S, ST>
+pub struct SimpleEventManager<I, ST>
 where
     I: Input,
     ST: Stats, //CE: CustomEvent<I, OT>,
@@ -23,33 +19,13 @@ where
     stats: ST,
     /// The events that happened since the last handle_in_broker
     events: Vec<Event<I>>,
-    phantom: PhantomData<S>,
 }
 
-impl<I, S, ST> EventManager<I, S> for SimpleEventManager<I, S, ST>
+impl<I, S, ST> EventFirer<I, S> for SimpleEventManager<I, ST>
 where
     I: Input,
     ST: Stats, //CE: CustomEvent<I, OT>,
 {
-    fn process<CS, E, OT>(
-        &mut self,
-        state: &mut S,
-        _executor: &mut E,
-        _scheduler: &CS,
-    ) -> Result<usize, Error>
-    where
-        CS: CorpusScheduler<I, S>,
-        E: Executor<I> + HasObservers<OT>,
-        OT: ObserversTuple,
-    {
-        let count = self.events.len();
-        while !self.events.is_empty() {
-            let event = self.events.pop().unwrap();
-            self.handle_in_client(state, event)?;
-        }
-        Ok(count)
-    }
-
     fn fire(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
         match Self::handle_in_broker(&mut self.stats, &event)? {
             BrokerEventResult::Forward => self.events.push(event),
@@ -59,7 +35,41 @@ where
     }
 }
 
-impl<I, S, ST> SimpleEventManager<I, S, ST>
+impl<I, S, ST> EventRestarter<S> for SimpleEventManager<I, ST>
+where
+    I: Input,
+    ST: Stats, //CE: CustomEvent<I, OT>,
+{
+}
+
+impl<E, I, S, ST, Z> EventProcessor<E, S, Z> for SimpleEventManager<I, ST>
+where
+    I: Input,
+    ST: Stats, //CE: CustomEvent<I, OT>,
+{
+    fn process(
+        &mut self,
+        _fuzzer: &mut Z,
+        state: &mut S,
+        _executor: &mut E,
+    ) -> Result<usize, Error> {
+        let count = self.events.len();
+        while !self.events.is_empty() {
+            let event = self.events.pop().unwrap();
+            self.handle_in_client(state, event)?;
+        }
+        Ok(count)
+    }
+}
+
+impl<E, I, S, ST, Z> EventManager<E, I, S, Z> for SimpleEventManager<I, ST>
+where
+    I: Input,
+    ST: Stats, //CE: CustomEvent<I, OT>,
+{
+}
+
+impl<I, ST> SimpleEventManager<I, ST>
 where
     I: Input,
     ST: Stats, //TODO CE: CustomEvent,
@@ -69,7 +79,6 @@ where
         Self {
             stats,
             events: vec![],
-            phantom: PhantomData,
         }
     }
 
@@ -141,7 +150,7 @@ where
 
     // Handle arriving events in the client
     #[allow(clippy::needless_pass_by_value, clippy::unused_self)]
-    fn handle_in_client(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
+    fn handle_in_client<S>(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
         Err(Error::Unknown(format!(
             "Received illegal message that message should not have arrived: {:?}.",
             event
