@@ -1,33 +1,36 @@
 //! The [`InProcessExecutor`] is a libfuzzer-like executor, that will simply call a function.
 //! It should usually be paired with extra error-handling, such as a restarting event manager, to be effective.
 
+use core::marker::PhantomData;
+
+#[cfg(any(unix, all(windows, feature = "std")))]
 use core::{
     ffi::c_void,
-    marker::PhantomData,
     ptr::{self, write_volatile},
     sync::atomic::{compiler_fence, Ordering},
 };
 
 #[cfg(unix)]
 use crate::bolts::os::unix_signals::setup_signal_handler;
-#[cfg(windows)]
+#[cfg(all(windows, feature = "std"))]
 use crate::bolts::os::windows_exceptions::setup_exception_handler;
 
 use crate::{
     corpus::Corpus,
-    events::EventManager,
+    events::{EventFirer, EventRestarter},
     executors::{
         Executor, ExitKind, HasExecHooks, HasExecHooksTuple, HasObservers, HasObserversHooks,
     },
     feedbacks::Feedback,
+    fuzzer::HasObjective,
     inputs::{HasTargetBytes, Input},
     observers::ObserversTuple,
-    state::{HasObjective, HasSolutions},
+    state::HasSolutions,
     Error,
 };
 
 /// The inmem executor simply calls a target function, then returns afterwards.
-pub struct InProcessExecutor<'a, EM, H, I, OT, S>
+pub struct InProcessExecutor<'a, H, I, OT, S>
 where
     H: FnMut(&[u8]) -> ExitKind,
     I: Input + HasTargetBytes,
@@ -37,10 +40,10 @@ where
     harness_fn: &'a mut H,
     /// The observers, observing each run
     observers: OT,
-    phantom: PhantomData<(EM, I, S)>,
+    phantom: PhantomData<(I, S)>,
 }
 
-impl<'a, EM, H, I, OT, S> Executor<I> for InProcessExecutor<'a, EM, H, I, OT, S>
+impl<'a, H, I, OT, S> Executor<I> for InProcessExecutor<'a, H, I, OT, S>
 where
     H: FnMut(&[u8]) -> ExitKind,
     I: Input + HasTargetBytes,
@@ -54,14 +57,20 @@ where
     }
 }
 
-impl<'a, EM, H, I, OT, S> HasExecHooks<EM, I, S> for InProcessExecutor<'a, EM, H, I, OT, S>
+impl<'a, EM, H, I, OT, S, Z> HasExecHooks<EM, I, S, Z> for InProcessExecutor<'a, H, I, OT, S>
 where
     H: FnMut(&[u8]) -> ExitKind,
     I: Input + HasTargetBytes,
     OT: ObserversTuple,
 {
     #[inline]
-    fn pre_exec(&mut self, _state: &mut S, _event_mgr: &mut EM, _input: &I) -> Result<(), Error> {
+    fn pre_exec(
+        &mut self,
+        _fuzzer: &mut Z,
+        _state: &mut S,
+        _event_mgr: &mut EM,
+        _input: &I,
+    ) -> Result<(), Error> {
         #[cfg(unix)]
         unsafe {
             let data = &mut unix_signal_handler::GLOBAL_STATE;
@@ -77,9 +86,10 @@ where
             // Since the state and event may have moved in memory, refresh them right before the signal may happen
             write_volatile(&mut data.state_ptr, _state as *mut _ as *mut c_void);
             write_volatile(&mut data.event_mgr_ptr, _event_mgr as *mut _ as *mut c_void);
+            write_volatile(&mut data.fuzzer_ptr, _fuzzer as *mut _ as *mut c_void);
             compiler_fence(Ordering::SeqCst);
         }
-        #[cfg(windows)]
+        #[cfg(all(windows, feature = "std"))]
         unsafe {
             let data = &mut windows_exception_handler::GLOBAL_STATE;
             write_volatile(
@@ -94,13 +104,20 @@ where
             // Since the state and event may have moved in memory, refresh them right before the signal may happen
             write_volatile(&mut data.state_ptr, _state as *mut _ as *mut c_void);
             write_volatile(&mut data.event_mgr_ptr, _event_mgr as *mut _ as *mut c_void);
+            write_volatile(&mut data.fuzzer_ptr, _fuzzer as *mut _ as *mut c_void);
             compiler_fence(Ordering::SeqCst);
         }
         Ok(())
     }
 
     #[inline]
-    fn post_exec(&mut self, _state: &mut S, _event_mgr: &mut EM, _input: &I) -> Result<(), Error> {
+    fn post_exec(
+        &mut self,
+        _fuzzer: &mut Z,
+        _state: &mut S,
+        _event_mgr: &mut EM,
+        _input: &I,
+    ) -> Result<(), Error> {
         #[cfg(unix)]
         unsafe {
             write_volatile(
@@ -109,7 +126,7 @@ where
             );
             compiler_fence(Ordering::SeqCst);
         }
-        #[cfg(windows)]
+        #[cfg(all(windows, feature = "std"))]
         unsafe {
             write_volatile(
                 &mut windows_exception_handler::GLOBAL_STATE.current_input_ptr,
@@ -121,7 +138,7 @@ where
     }
 }
 
-impl<'a, EM, H, I, OT, S> HasObservers<OT> for InProcessExecutor<'a, EM, H, I, OT, S>
+impl<'a, H, I, OT, S> HasObservers<OT> for InProcessExecutor<'a, H, I, OT, S>
 where
     H: FnMut(&[u8]) -> ExitKind,
     I: Input + HasTargetBytes,
@@ -138,15 +155,16 @@ where
     }
 }
 
-impl<'a, EM, H, I, OT, S> HasObserversHooks<EM, I, OT, S> for InProcessExecutor<'a, EM, H, I, OT, S>
+impl<'a, EM, H, I, OT, S, Z> HasObserversHooks<EM, I, OT, S, Z>
+    for InProcessExecutor<'a, H, I, OT, S>
 where
     H: FnMut(&[u8]) -> ExitKind,
     I: Input + HasTargetBytes,
-    OT: ObserversTuple + HasExecHooksTuple<EM, I, S>,
+    OT: ObserversTuple + HasExecHooksTuple<EM, I, S, Z>,
 {
 }
 
-impl<'a, EM, H, I, OT, S> InProcessExecutor<'a, EM, H, I, OT, S>
+impl<'a, H, I, OT, S> InProcessExecutor<'a, H, I, OT, S>
 where
     H: FnMut(&[u8]) -> ExitKind,
     I: Input + HasTargetBytes,
@@ -158,43 +176,45 @@ where
     /// * `harness_fn` - the harness, executiong the function
     /// * `observers` - the observers observing the target during execution
     /// This may return an error on unix, if signal handler setup fails
-    pub fn new<OC, OF>(
+    pub fn new<EM, OC, OF, Z>(
         harness_fn: &'a mut H,
         observers: OT,
+        _fuzzer: &mut Z,
         _state: &mut S,
         _event_mgr: &mut EM,
     ) -> Result<Self, Error>
     where
-        EM: EventManager<I, S>,
+        EM: EventFirer<I, S> + EventRestarter<S>,
         OC: Corpus<I>,
-        OF: Feedback<I>,
-        S: HasObjective<OF, I> + HasSolutions<OC, I>,
+        OF: Feedback<I, S>,
+        S: HasSolutions<OC, I>,
+        Z: HasObjective<I, OF, S>,
     {
         #[cfg(unix)]
         unsafe {
             let data = &mut unix_signal_handler::GLOBAL_STATE;
             write_volatile(
                 &mut data.crash_handler,
-                unix_signal_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S>,
+                unix_signal_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S, Z>,
             );
             write_volatile(
                 &mut data.timeout_handler,
-                unix_signal_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S>,
+                unix_signal_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S, Z>,
             );
 
             setup_signal_handler(data)?;
             compiler_fence(Ordering::SeqCst);
         }
-        #[cfg(windows)]
+        #[cfg(all(windows, feature = "std"))]
         unsafe {
             let data = &mut windows_exception_handler::GLOBAL_STATE;
             write_volatile(
                 &mut data.crash_handler,
-                windows_exception_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S>,
+                windows_exception_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S, Z>,
             );
             //write_volatile(
             //    &mut data.timeout_handler,
-            //    windows_exception_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S>,
+            //    windows_exception_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S, Z>,
             //);
 
             setup_exception_handler(data)?;
@@ -211,7 +231,7 @@ where
     /// Retrieve the harness function.
     #[inline]
     pub fn harness(&self) -> &H {
-        self.harness_fn
+        &self.harness_fn
     }
 
     /// Retrieve the harness function for a mutable reference.
@@ -232,12 +252,13 @@ mod unix_signal_handler {
     use crate::{
         bolts::os::unix_signals::{Handler, Signal},
         corpus::{Corpus, Testcase},
-        events::{Event, EventManager},
+        events::{Event, EventFirer, EventRestarter},
         executors::ExitKind,
         feedbacks::Feedback,
+        fuzzer::HasObjective,
         inputs::{HasTargetBytes, Input},
         observers::ObserversTuple,
-        state::{HasObjective, HasSolutions},
+        state::HasSolutions,
     };
 
     // TODO merge GLOBAL_STATE with the Windows one
@@ -248,6 +269,8 @@ mod unix_signal_handler {
         state_ptr: ptr::null_mut(),
         /// The event manager ptr for signal handling
         event_mgr_ptr: ptr::null_mut(),
+        /// The fuzzer ptr for signal handling
+        fuzzer_ptr: ptr::null_mut(),
         /// The observers ptr for signal handling
         observers_ptr: ptr::null(),
         /// The current input for signal handling
@@ -261,6 +284,7 @@ mod unix_signal_handler {
     pub struct InProcessExecutorHandlerData {
         pub state_ptr: *mut c_void,
         pub event_mgr_ptr: *mut c_void,
+        pub fuzzer_ptr: *mut c_void,
         pub observers_ptr: *const c_void,
         pub current_input_ptr: *const c_void,
         pub crash_handler: unsafe fn(Signal, siginfo_t, &mut ucontext_t, data: &mut Self),
@@ -308,21 +332,23 @@ mod unix_signal_handler {
     }
 
     #[cfg(unix)]
-    pub unsafe fn inproc_timeout_handler<EM, I, OC, OF, OT, S>(
+    pub unsafe fn inproc_timeout_handler<EM, I, OC, OF, OT, S, Z>(
         _signal: Signal,
         _info: siginfo_t,
         _context: &mut ucontext_t,
         data: &mut InProcessExecutorHandlerData,
     ) where
-        EM: EventManager<I, S>,
+        EM: EventFirer<I, S> + EventRestarter<S>,
         OT: ObserversTuple,
         OC: Corpus<I>,
-        OF: Feedback<I>,
-        S: HasObjective<OF, I> + HasSolutions<OC, I>,
+        OF: Feedback<I, S>,
+        S: HasSolutions<OC, I>,
         I: Input + HasTargetBytes,
+        Z: HasObjective<I, OF, S>,
     {
         let state = (data.state_ptr as *mut S).as_mut().unwrap();
         let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
+        let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
         let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
 
         if data.current_input_ptr.is_null() {
@@ -332,20 +358,21 @@ mod unix_signal_handler {
             #[cfg(feature = "std")]
             println!("Timeout in fuzz run.");
             #[cfg(feature = "std")]
-            let _ = stdout().flush();
+            let _res = stdout().flush();
 
             let input = (data.current_input_ptr as *const I).as_ref().unwrap();
             data.current_input_ptr = ptr::null();
 
-            let interesting = state
+            let interesting = fuzzer
                 .objective_mut()
-                .is_interesting(&input, observers, &ExitKind::Timeout)
+                .is_interesting(state, &input, observers, &ExitKind::Timeout)
                 .expect("In timeout handler objective failure.");
+
             if interesting {
                 let mut new_testcase = Testcase::new(input.clone());
-                state
+                fuzzer
                     .objective_mut()
-                    .append_metadata(&mut new_testcase)
+                    .append_metadata(state, &mut new_testcase)
                     .expect("Failed adding metadata");
                 state
                     .solutions_mut()
@@ -379,18 +406,19 @@ mod unix_signal_handler {
     /// Will be used for signal handling.
     /// It will store the current State to shmem, then exit.
     #[allow(clippy::too_many_lines)]
-    pub unsafe fn inproc_crash_handler<EM, I, OC, OF, OT, S>(
+    pub unsafe fn inproc_crash_handler<EM, I, OC, OF, OT, S, Z>(
         _signal: Signal,
         _info: siginfo_t,
         _context: &mut ucontext_t,
         data: &mut InProcessExecutorHandlerData,
     ) where
-        EM: EventManager<I, S>,
+        EM: EventFirer<I, S> + EventRestarter<S>,
         OT: ObserversTuple,
         OC: Corpus<I>,
-        OF: Feedback<I>,
-        S: HasObjective<OF, I> + HasSolutions<OC, I>,
+        OF: Feedback<I, S>,
+        S: HasSolutions<OC, I>,
         I: Input + HasTargetBytes,
+        Z: HasObjective<I, OF, S>,
     {
         #[cfg(all(target_os = "android", target_arch = "aarch64"))]
         let _context = *(((_context as *mut _ as *mut c_void as usize) + 128) as *mut c_void
@@ -433,6 +461,7 @@ mod unix_signal_handler {
         } else {
             let state = (data.state_ptr as *mut S).as_mut().unwrap();
             let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
+            let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
             let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
 
             #[cfg(feature = "std")]
@@ -444,7 +473,7 @@ mod unix_signal_handler {
                 target_arch = "aarch64"
             ))]
             {
-                use crate::utils::find_mapping_for_address;
+                use crate::bolts::os::find_mapping_for_address;
                 println!("{:━^100}", " CRASH ");
                 println!(
                     "Received signal {} at 0x{:016x}, fault address: 0x{:016x}",
@@ -477,22 +506,23 @@ mod unix_signal_handler {
             }
 
             #[cfg(feature = "std")]
-            let _ = stdout().flush();
+            let _res = stdout().flush();
 
             let input = (data.current_input_ptr as *const I).as_ref().unwrap();
             // Make sure we don't crash in the crash handler forever.
             data.current_input_ptr = ptr::null();
 
-            let interesting = state
+            let interesting = fuzzer
                 .objective_mut()
-                .is_interesting(&input, observers, &ExitKind::Crash)
+                .is_interesting(state, &input, observers, &ExitKind::Crash)
                 .expect("In crash handler objective failure.");
+
             if interesting {
                 let new_input = input.clone();
                 let mut new_testcase = Testcase::new(new_input);
-                state
+                fuzzer
                     .objective_mut()
-                    .append_metadata(&mut new_testcase)
+                    .append_metadata(state, &mut new_testcase)
                     .expect("Failed adding metadata");
                 state
                     .solutions_mut()
@@ -521,7 +551,7 @@ mod unix_signal_handler {
     }
 }
 
-#[cfg(windows)]
+#[cfg(all(windows, feature = "std"))]
 mod windows_exception_handler {
     use alloc::vec::Vec;
     use core::{ffi::c_void, ptr};
@@ -536,12 +566,13 @@ mod windows_exception_handler {
             },
         },
         corpus::{Corpus, Testcase},
-        events::{Event, EventManager},
+        events::{Event, EventFirer, EventRestarter},
         executors::ExitKind,
         feedbacks::Feedback,
+        fuzzer::HasObjective,
         inputs::{HasTargetBytes, Input},
         observers::ObserversTuple,
-        state::{HasObjective, HasSolutions},
+        state::HasSolutions,
     };
 
     /// Signal handling on unix systems needs some nasty unsafe.
@@ -550,6 +581,8 @@ mod windows_exception_handler {
         state_ptr: ptr::null_mut(),
         /// The event manager ptr for signal handling
         event_mgr_ptr: ptr::null_mut(),
+        /// The fuzzer ptr for signal handling
+        fuzzer_ptr: ptr::null_mut(),
         /// The observers ptr for signal handling
         observers_ptr: ptr::null(),
         /// The current input for signal handling
@@ -563,6 +596,7 @@ mod windows_exception_handler {
     pub struct InProcessExecutorHandlerData {
         pub state_ptr: *mut c_void,
         pub event_mgr_ptr: *mut c_void,
+        pub fuzzer_ptr: *mut c_void,
         pub observers_ptr: *const c_void,
         pub current_input_ptr: *const c_void,
         pub crash_handler: unsafe fn(ExceptionCode, *mut EXCEPTION_POINTERS, &mut Self),
@@ -592,23 +626,25 @@ mod windows_exception_handler {
         }
     }
 
-    pub unsafe fn inproc_crash_handler<EM, I, OC, OF, OT, S>(
+    pub unsafe fn inproc_crash_handler<EM, I, OC, OF, OT, S, Z>(
         code: ExceptionCode,
         exception_pointers: *mut EXCEPTION_POINTERS,
         data: &mut InProcessExecutorHandlerData,
     ) where
-        EM: EventManager<I, S>,
+        EM: EventFirer<I, S> + EventRestarter<S>,
         OT: ObserversTuple,
         OC: Corpus<I>,
-        OF: Feedback<I>,
-        S: HasObjective<OF, I> + HasSolutions<OC, I>,
+        OF: Feedback<I, S>,
+        S: HasSolutions<OC, I>,
         I: Input + HasTargetBytes,
+        Z: HasObjective<I, OF, S>,
     {
         #[cfg(feature = "std")]
         println!("Crashed with {}", code);
         if !data.current_input_ptr.is_null() {
             let state = (data.state_ptr as *mut S).as_mut().unwrap();
             let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
+            let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
             let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
 
             #[cfg(feature = "std")]
@@ -620,16 +656,17 @@ mod windows_exception_handler {
             // Make sure we don't crash in the crash handler forever.
             data.current_input_ptr = ptr::null();
 
-            let interesting = state
+            let interesting = fuzzer
                 .objective_mut()
-                .is_interesting(&input, observers, &ExitKind::Crash)
+                .is_interesting(state, &input, observers, &ExitKind::Crash)
                 .expect("In crash handler objective failure.");
+
             if interesting {
                 let new_input = input.clone();
                 let mut new_testcase = Testcase::new(new_input);
-                state
+                fuzzer
                     .objective_mut()
-                    .append_metadata(&mut new_testcase)
+                    .append_metadata(state, &mut new_testcase)
                     .expect("Failed adding metadata");
                 state
                     .solutions_mut()
@@ -688,7 +725,6 @@ mod windows_exception_handler {
 
 #[cfg(test)]
 mod tests {
-
     use core::marker::PhantomData;
 
     use crate::{
@@ -701,7 +737,7 @@ mod tests {
     fn test_inmem_exec() {
         let mut harness = |_buf: &[u8]| ExitKind::Ok;
 
-        let mut in_process_executor = InProcessExecutor::<(), _, NopInput, (), ()> {
+        let mut in_process_executor = InProcessExecutor::<_, NopInput, (), ()> {
             harness_fn: &mut harness,
             observers: tuple_list!(),
             phantom: PhantomData,
