@@ -35,52 +35,100 @@ pub enum ExitKind {
 }
 
 /// Pre and post exec hooks
-pub trait HasExecHooks<EM, I, S> {
+pub trait HasExecHooks<EM, I, S, Z> {
     /// Called right before exexution starts
     #[inline]
-    fn pre_exec(&mut self, _state: &mut S, _mgr: &mut EM, _input: &I) -> Result<(), Error> {
+    fn pre_exec(
+        &mut self,
+        _fuzzer: &mut Z,
+        _state: &mut S,
+        _mgr: &mut EM,
+        _input: &I,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
     /// Called right after execution finished.
     #[inline]
-    fn post_exec(&mut self, _state: &mut S, _mgr: &mut EM, _input: &I) -> Result<(), Error> {
+    fn post_exec(
+        &mut self,
+        _fuzzer: &mut Z,
+        _state: &mut S,
+        _mgr: &mut EM,
+        _input: &I,
+    ) -> Result<(), Error> {
         Ok(())
     }
 }
 
 /// A haskell-style tuple of objects that have pre and post exec hooks
-pub trait HasExecHooksTuple<EM, I, S> {
+pub trait HasExecHooksTuple<EM, I, S, Z> {
     /// This is called right before the next execution.
-    fn pre_exec_all(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error>;
+    fn pre_exec_all(
+        &mut self,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<(), Error>;
 
     /// This is called right after the last execution
-    fn post_exec_all(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error>;
+    fn post_exec_all(
+        &mut self,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<(), Error>;
 }
 
-impl<EM, I, S> HasExecHooksTuple<EM, I, S> for () {
-    fn pre_exec_all(&mut self, _state: &mut S, _mgr: &mut EM, _input: &I) -> Result<(), Error> {
+impl<EM, I, S, Z> HasExecHooksTuple<EM, I, S, Z> for () {
+    fn pre_exec_all(
+        &mut self,
+        _fuzzer: &mut Z,
+        _state: &mut S,
+        _mgr: &mut EM,
+        _input: &I,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    fn post_exec_all(&mut self, _state: &mut S, _mgr: &mut EM, _input: &I) -> Result<(), Error> {
+    fn post_exec_all(
+        &mut self,
+        _fuzzer: &mut Z,
+        _state: &mut S,
+        _mgr: &mut EM,
+        _input: &I,
+    ) -> Result<(), Error> {
         Ok(())
     }
 }
 
-impl<EM, I, S, Head, Tail> HasExecHooksTuple<EM, I, S> for (Head, Tail)
+impl<EM, I, S, Z, Head, Tail> HasExecHooksTuple<EM, I, S, Z> for (Head, Tail)
 where
-    Head: HasExecHooks<EM, I, S>,
-    Tail: HasExecHooksTuple<EM, I, S>,
+    Head: HasExecHooks<EM, I, S, Z>,
+    Tail: HasExecHooksTuple<EM, I, S, Z>,
 {
-    fn pre_exec_all(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error> {
-        self.0.pre_exec(state, mgr, input)?;
-        self.1.pre_exec_all(state, mgr, input)
+    fn pre_exec_all(
+        &mut self,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<(), Error> {
+        self.0.pre_exec(fuzzer, state, mgr, input)?;
+        self.1.pre_exec_all(fuzzer, state, mgr, input)
     }
 
-    fn post_exec_all(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error> {
-        self.0.post_exec(state, mgr, input)?;
-        self.1.post_exec_all(state, mgr, input)
+    fn post_exec_all(
+        &mut self,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<(), Error> {
+        self.0.post_exec(fuzzer, state, mgr, input)?;
+        self.1.post_exec_all(fuzzer, state, mgr, input)
     }
 }
 
@@ -97,20 +145,33 @@ where
 }
 
 /// Execute the exec hooks of the observers if they all implement [`HasExecHooks`].
-pub trait HasObserversHooks<EM, I, OT, S>: HasObservers<OT>
+pub trait HasObserversHooks<EM, I, OT, S, Z>: HasObservers<OT>
 where
-    OT: ObserversTuple + HasExecHooksTuple<EM, I, S>,
+    OT: ObserversTuple + HasExecHooksTuple<EM, I, S, Z>,
 {
     /// Run the pre exec hook for all [`crate::observers::Observer`]`s` linked to this [`Executor`].
     #[inline]
-    fn pre_exec_observers(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error> {
-        self.observers_mut().pre_exec_all(state, mgr, input)
+    fn pre_exec_observers(
+        &mut self,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<(), Error> {
+        self.observers_mut().pre_exec_all(fuzzer, state, mgr, input)
     }
 
     /// Run the post exec hook for all the [`crate::observers::Observer`]`s` linked to this [`Executor`].
     #[inline]
-    fn post_exec_observers(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error> {
-        self.observers_mut().post_exec_all(state, mgr, input)
+    fn post_exec_observers(
+        &mut self,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<(), Error> {
+        self.observers_mut()
+            .post_exec_all(fuzzer, state, mgr, input)
     }
 }
 
@@ -142,7 +203,8 @@ where
     }
 }
 
-impl<EM, I, S> HasExecHooks<EM, I, S> for NopExecutor<EM, I, S> where I: Input + HasTargetBytes {}
+impl<EM, I, S, Z> HasExecHooks<EM, I, S, Z> for NopExecutor<EM, I, S> where I: Input + HasTargetBytes
+{}
 
 #[cfg(test)]
 mod test {
