@@ -1,12 +1,11 @@
 //! A wide variety of mutations used during fuzzing.
 
 use crate::{
-    bolts::tuples::Named,
+    bolts::{rands::Rand, tuples::Named},
     corpus::Corpus,
     inputs::{HasBytesVec, Input},
     mutators::{MutationResult, Mutator},
     state::{HasCorpus, HasMaxSize, HasRand},
-    utils::Rand,
     Error,
 };
 
@@ -1832,13 +1831,14 @@ mod tests {
 
     use super::*;
     use crate::{
-        bolts::tuples::tuple_list,
-        bolts::tuples::HasLen,
+        bolts::{
+            rands::StdRand,
+            tuples::{tuple_list, HasLen},
+        },
         corpus::{Corpus, InMemoryCorpus},
         inputs::BytesInput,
         mutators::MutatorsTuple,
-        state::{HasMetadata, State},
-        utils::StdRand,
+        state::{HasMetadata, StdState},
     };
 
     fn test_mutations<C, I, R, S>() -> impl MutatorsTuple<I, S>
@@ -1895,7 +1895,7 @@ mod tests {
             .add(BytesInput::new(vec![0x42; 0x1337]).into())
             .unwrap();
 
-        let mut state = State::new(rand, corpus, (), InMemoryCorpus::new(), ());
+        let mut state = StdState::new(rand, corpus, InMemoryCorpus::new(), ());
 
         let mut mutations = test_mutations();
         for _ in 0..2 {
