@@ -1,7 +1,6 @@
 //! The `ScheduledMutator` schedules multiple mutations internally.
 
-use alloc::string::String;
-use alloc::vec::Vec;
+use alloc::{string::String, vec::Vec};
 use core::{
     fmt::{self, Debug},
     marker::PhantomData,
@@ -9,12 +8,15 @@ use core::{
 use serde::{Deserialize, Serialize};
 
 use crate::{
-    bolts::tuples::{tuple_list, NamedTuple},
+    bolts::{
+        rands::Rand,
+        tuples::{tuple_list, NamedTuple},
+        AsSlice,
+    },
     corpus::Corpus,
     inputs::{HasBytesVec, Input},
     mutators::{MutationResult, Mutator, MutatorsTuple},
     state::{HasCorpus, HasMaxSize, HasMetadata, HasRand},
-    utils::{AsSlice, Rand},
     Error,
 };
 
@@ -397,6 +399,7 @@ where
 #[cfg(test)]
 mod tests {
     use crate::{
+        bolts::rands::{Rand, StdRand, XkcdRand},
         corpus::{Corpus, InMemoryCorpus, Testcase},
         inputs::{BytesInput, HasBytesVec},
         mutators::{
@@ -404,8 +407,7 @@ mod tests {
             scheduled::{havoc_mutations, StdScheduledMutator},
             Mutator,
         },
-        state::State,
-        utils::{Rand, StdRand, XkcdRand},
+        state::StdState,
     };
 
     #[test]
@@ -419,7 +421,7 @@ mod tests {
         let testcase = corpus.get(0).expect("Corpus did not contain entries");
         let mut input = testcase.borrow_mut().load_input().unwrap().clone();
 
-        let mut state = State::new(rand, corpus, (), InMemoryCorpus::new(), ());
+        let mut state = StdState::new(rand, corpus, InMemoryCorpus::new(), ());
 
         rand.set_seed(5);
 
@@ -446,7 +448,7 @@ mod tests {
         let mut input = testcase.borrow_mut().load_input().unwrap().clone();
         let input_prior = input.clone();
 
-        let mut state = State::new(rand, corpus, (), InMemoryCorpus::new(), ());
+        let mut state = StdState::new(rand, corpus, InMemoryCorpus::new(), ());
 
         let mut havoc = StdScheduledMutator::new(havoc_mutations());
 
