@@ -11,90 +11,67 @@ pub use mutational::{MutationalStage, StdMutationalStage};
 
 //pub mod power;
 //pub use power::PowerMutationalStage;
-
-use crate::{
-    bolts::tuples::TupleList, events::EventManager, executors::Executor, inputs::Input,
-    state::HasClientPerfStats, Error,
-};
+use crate::Error;
 
 /// A stage is one step in the fuzzing process.
 /// Multiple stages will be scheduled one by one for each input.
-pub trait Stage<CS, E, EM, I, S>
-where
-    EM: EventManager<I, S>,
-    E: Executor<I>,
-    I: Input,
-{
+pub trait Stage<E, EM, S, Z> {
     /// Run the stage
     fn perform(
         &mut self,
-        state: &mut S,
+        fuzzer: &mut Z,
         executor: &mut E,
+        state: &mut S,
         manager: &mut EM,
-        scheduler: &CS,
         corpus_idx: usize,
     ) -> Result<(), Error>;
 }
 
 /// A tuple holding all `Stages` used for fuzzing.
-pub trait StagesTuple<CS, E, EM, I, S>
-where
-    EM: EventManager<I, S>,
-    E: Executor<I>,
-    I: Input,
-{
+pub trait StagesTuple<E, EM, S, Z> {
     /// Performs all `Stages` in this tuple
     fn perform_all(
         &mut self,
-        state: &mut S,
+        fuzzer: &mut Z,
         executor: &mut E,
+        state: &mut S,
         manager: &mut EM,
-        scheduler: &CS,
         corpus_idx: usize,
     ) -> Result<(), Error>;
 }
 
-impl<CS, E, EM, I, S> StagesTuple<CS, E, EM, I, S> for ()
-where
-    EM: EventManager<I, S>,
-    E: Executor<I>,
-    I: Input,
-{
+impl<E, EM, S, Z> StagesTuple<E, EM, S, Z> for () {
     fn perform_all(
         &mut self,
-        _: &mut S,
+        _: &mut Z,
         _: &mut E,
+        _: &mut S,
         _: &mut EM,
-        _: &CS,
         _: usize,
     ) -> Result<(), Error> {
         Ok(())
     }
 }
 
-impl<Head, Tail, CS, E, EM, I, S> StagesTuple<CS, E, EM, I, S> for (Head, Tail)
+impl<Head, Tail, E, EM, S, Z> StagesTuple<E, EM, S, Z> for (Head, Tail)
 where
-    Head: Stage<CS, E, EM, I, S>,
-    Tail: StagesTuple<CS, E, EM, I, S> + TupleList,
-    EM: EventManager<I, S>,
-    E: Executor<I>,
-    I: Input,
-    S: HasClientPerfStats,
+    Head: Stage<E, EM, S, Z>,
+    Tail: StagesTuple<E, EM, S, Z>,
 {
     fn perform_all(
         &mut self,
-        state: &mut S,
+        fuzzer: &mut Z,
         executor: &mut E,
+        state: &mut S,
         manager: &mut EM,
-        scheduler: &CS,
         corpus_idx: usize,
     ) -> Result<(), Error> {
         // Perform the current stage
         self.0
-            .perform(state, executor, manager, scheduler, corpus_idx)?;
+            .perform(fuzzer, executor, state, manager, corpus_idx)?;
 
         // Execute the remaining stages
         self.1
-            .perform_all(state, executor, manager, scheduler, corpus_idx)
+            .perform_all(fuzzer, executor, state, manager, corpus_idx)
     }
 }
